package com.gct.dataengine.ossUtil;

import com.gct.common.util.StringUtil;
import io.minio.*;
import io.minio.http.Method;
import io.minio.messages.DeleteError;
import io.minio.messages.DeleteObject;
import io.minio.messages.Item;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * class name: MinioUtil
 *
 * @author gxt
 * @version 1.0
 * @since 2022/5/25 下午3:41 周三
 */
@Component
public class MinioUtil {

    @Autowired
    MinioClient minioClient;

    /***
     * 通过文件流上传
     * @param bucketName 桶名(储存在哪里)
     * @param objectName  文件名字(文件在minio上的名字)
     * @param inputStream  文件流
     * @param cover 文件是否覆盖
     * @return 是否成功
     */
    public String uploadByInputStream(String bucketName, String objectName, InputStream inputStream, String contentType, String cover) {
        if (StringUtils.isEmpty(cover)) cover = "false";
        if (statObject(bucketName, objectName) && !Boolean.parseBoolean(cover)) {
            return "exist-file";
        }
        return putObject(bucketName, objectName, inputStream, contentType) ? "success" : "failed";
    }

    /***
     * 通过文件地址上传
     * @param bucketName 桶名(储存在哪里)
     * @param objectName  文件名字(文件在minio上的名字)
     * @param filePath  文件的地址
     * @param cover 文件是否覆盖
     * @return 是否成功
     */
    public String uploadByFilePath(String bucketName, String objectName, String filePath, String contentType, String cover) {
        if (StringUtils.isEmpty(cover)) cover = "false";
        if (statObject(bucketName, objectName) && !Boolean.parseBoolean(cover)) {
            return "该文件在云上已经存在,请更换名称或者选择强制覆盖";
        }
        return uploadObject(bucketName, objectName, filePath, contentType) ? "success" : "failed";
    }

    /***
     * 图片预览或者文件下载
     * @param bucketName 桶名
     * @param objectName 文件名
     * @return 预览或者下载url
     */
    public String previewImgOrDownLoadFile(String bucketName, String objectName) {
        try {
            //todo 这里现在有一个问题,过期时间最长是7天,但是预览并不需要过期
            return minioClient.getPresignedObjectUrl(GetPresignedObjectUrlArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .method(Method.GET)
                    .expiry(7, TimeUnit.DAYS).build());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    public InputStream getObject(String bucketName, String objectName) {
        try {
            return minioClient.getObject(GetObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .build());
        } catch (Exception ignored) {
            return null;
        }
    }

    public void downloadObject(String bucketName, String objectName) {
        try {
            minioClient.downloadObject(DownloadObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .build());
        } catch (Exception ignored) {

        }
    }

    /***
     * 获取分片上传的地址
     * @param bucketName 桶名
     * @param objectName 文件名
     * @return 有权限分片上传的url
     */
    public String getPartUploadUrl(String bucketName, String objectName) {
        try {
            return minioClient.getPresignedObjectUrl(GetPresignedObjectUrlArgs.builder()
                    .method(Method.PUT).bucket(bucketName).object(objectName).build());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    /***
     * 分片上传之后按照规则合并
     * @param bucketName 桶名
     * @param objectName 合并后的文件名字
     * @param sources 合并的文件
     * @return 是否成功
     */
    public boolean composeObjects(String bucketName, String objectName, List<ComposeSource> sources) {
        makeBucket(bucketName);
        try {
            ObjectWriteResponse response = minioClient.composeObject(ComposeObjectArgs.builder().bucket(bucketName).object(objectName).sources(sources).build());
            if (Objects.nonNull(response.etag())) return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /***
     * 复制文件
     * @param sourceBucket 源桶名
     * @param sourceObject 源文件名
     * @param targetBucket 目标桶名
     * @param targetObject 目标文件名
     * @return 复制成功或失败
     */
    public boolean copyObjects(String sourceBucket,String sourceObject,String targetBucket,String targetObject){
       try {
           ObjectWriteResponse copyObject = minioClient.copyObject(CopyObjectArgs.builder()
                   .bucket(targetBucket).object(targetObject)
                   .source(CopySource.builder().bucket(sourceBucket).object(sourceObject).build())
                   .build());
         if (!StringUtil.isBlankOrEmpty(copyObject.etag())) return true;
       }catch (Exception e){
           e.printStackTrace();
           return false;
       }
       return false;
    }

    /***
     * 列出所有相关的文件内容
     * @param bucketName 桶名
     * @param prefix 文件的前缀,就是分片上传时统一的前缀
     * @return 所有符合要求的数据集合
     */
    public List<String> getListObjects(String bucketName, String prefix) {
        makeBucket(bucketName);
        List<String> list = new ArrayList<>();
        try {
            Iterable<Result<Item>> results = minioClient.listObjects(ListObjectsArgs.builder().bucket(bucketName).prefix(prefix).build());
            for (Result<Item> result : results) {
                list.add(result.get().objectName());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return list;
    }

    public boolean deleteObjects(String bucketName, List<DeleteObject> objects) {
        makeBucket(bucketName);
        boolean res = true;
        try {
            Iterable<Result<DeleteError>> results = minioClient.removeObjects(RemoveObjectsArgs.builder().bucket(bucketName).objects(objects).build());
            for (Result<DeleteError> result : results) {
                String code = result.get().code();
                String objectName = result.get().objectName();
                String message = result.get().message();
                res = org.apache.commons.lang3.StringUtils.isEmpty(code) || org.apache.commons.lang3.StringUtils.isEmpty(objectName) || org.apache.commons.lang3.StringUtils.isEmpty(message);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return res;
    }

    public boolean statObject(String bucketName, String objectName) {
        try {
            minioClient.statObject(StatObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName).build());
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    private boolean putObject(String bucketName, String objectName, InputStream inputStream, String contentType) {
        try {
            makeBucket(bucketName);
            ObjectWriteResponse response = minioClient.putObject(PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .contentType(contentType)
                    .stream(inputStream, inputStream.available(), -1)
                    .build());
            if (Objects.nonNull(response.etag())) return true;
        } catch (Exception ignored) {
        }
        return false;
    }

    private boolean uploadObject(String bucketName, String objectName, String filePath, String contentType) {
        try {
            ObjectWriteResponse response = minioClient.uploadObject(UploadObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .contentType(contentType)
                    .filename(filePath).build());
            if (Objects.nonNull(response.etag())) return true;
        } catch (Exception ignored) {
        }
        return false;
    }

    private boolean bucketExists(String bucketName) {
        try {
            return minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    private void makeBucket(String bucketName) {
        try {
            if (!bucketExists(bucketName)) {
                minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /***
     *
     * @param bucketName 桶名称
     * @param objectName 对象名称
     * @param filePath 要写入的文件地址
     */
    public String getFileToPath(String bucketName, String objectName, String filePath) {
        makeBucket(bucketName);
        InputStream inputStream = null;
        BufferedInputStream fis = null;
        BufferedOutputStream bis = null;
        try {
            inputStream = minioClient.getObject(GetObjectArgs.builder()
                    .bucket(bucketName)
                    .object(objectName)
                    .build());
            fis = new BufferedInputStream(inputStream);
            bis = new BufferedOutputStream(Files.newOutputStream(new File(filePath+"/"+objectName).toPath()));
            byte[] buff = new byte[1024 * 5];
            int len;
            while ((len = fis.read(buff)) != -1) {
                bis.write(buff, 0, len);
            }
            fis.close();
            bis.close();
            return filePath+"/"+objectName;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (bis != null) bis.close();
                if (fis != null) fis.close();
                if (inputStream != null) inputStream.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return "";
    }
}
